/*
 * Copyright (c) 2014 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*global define, document, tau, require*/

/**
 * Galleries page module.
 *
 * @module views/galleriesPage
 * @requires {@link core/event}
 * @requires {@link core/template}
 * @requires {@link models/content}
 * @requires {@link views/ui}
 * @namespace views/galleriesPage
 * @memberof views
 * @author Dominik Koleda <d.koleda@samsung.com>
 * @author Piotr Dabrowski <p.dabrowski2@samsung.com>
 * @author Kamil Stepczuk <k.stepczuk@samsung.com>
 */

define({
    name: 'views/galleriesPage',
    requires: [
        'core/event',
        'core/template',
        'models/content',
        'views/ui'
    ],
    def: function galleriesPage(req) {
        'use strict';

        /**
         * Event module object.
         *
         * @memberof views/galleriesPage
         * @private
         * @type {Module}
         */
        var event = req.core.event,

            /**
             * Template module object.
             *
             * @memberof views/galleriesPage
             * @private
             * @type {Module}
             */
            template = req.core.template,

            /**
             * Content module object.
             *
             * @memberof views/galleriesPage
             * @private
             * @type {Module}
             */
            content = req.models.content,

            /**
             * Photos page module object.
             *
             * @memberof views/galleriesPage
             * @private
             * @type {Module}
             */
            photosPage = null,

            /**
             * Galleries page element.
             *
             * @memberof views/galleriesPage
             * @private
             * @type {HTMLElement}
             */
            page = null,

            /**
             * Reference to the gallery list element.
             *
             * @memberof views/galleriesPage
             * @private
             * @type {HTMLElement}
             */
            galleriesElement = null,

            /**
             * Reference to the message element.
             *
             * @memberof views/galleriesPage
             * @private
             * @type {HTMLElement}
             */
            emptyGalleryMsgElement = null;

        /**
         * Opens and shows gallery.
         *
         * @memberof views/galleriesPage
         * @private
         * @param {string} galleryPath
         */
        function openGallery(galleryPath) {
            if (photosPage.drawPhotos(galleryPath)) {
                photosPage.show();
            }
        }

        /**
         * Handles of click event.
         *
         * @memberof views/galleriesPage
         * @private
         * @param {Event} event
         */
        function onClickGalleries(event) {
            var target = event.target;

            if (target.classList.contains('list-image')) {
                openGallery(target.dataset.path);
            }
        }

        /**
         * Registers module event listeners.
         *
         * @memberof views/galleriesPage
         * @private
         */
        function bindEvents() {
            event.listeners({
                'models.content.refreshed': drawGalleries
            });
            galleriesElement.addEventListener('click', onClickGalleries);
        }

        /**
         * Draws galleries.
         *
         * @memberof views/galleriesPage
         * @public
         */
        function drawGalleries() {
            var galleries = content.getGalleries();

            if (galleries.length) {
                if (galleries.length === 1) {
                    page.classList.add('one-gallery');
                } else {
                    page.classList.remove('one-gallery');
                }

                galleriesElement.innerHTML =
                    template.get('galleries', {galleries: galleries});

                emptyGalleryMsgElement.classList.add('hidden');
                galleriesElement.classList.remove('hidden');
            } else {
                galleriesElement.classList.add('hidden');
                emptyGalleryMsgElement.classList.remove('hidden');
            }
        }

        /**
         * Shows galleries page.
         *
         * @memberof views/galleriesPage
         * @public
         */
        function show() {
            tau.changePage(page);
        }

        /**
         * Initializes module.
         *
         * @memberof views/galleriesPage
         * @public
         */
        function init() {
            photosPage = require('views/photosPage');
            page = document.getElementById('galleries-page');
            galleriesElement = document.getElementById('galleries-list');
            emptyGalleryMsgElement = document.getElementById('no-images-msg');
            bindEvents();
        }

        return {
            init: init,
            show: show,
            drawGalleries: drawGalleries
        };
    }
});
